package org.eso.oca.data;


/**
 * Class MetaTable
 *
 * Business object representing a row on obs_tables table from raw database, 
 * or a row on calib_tables from calib database 
 *
 * @version $Revision: 1.4 $
 * @author pnunes, Mar 30, 2005
 */

public class MetaTable {
    private String name;
    private boolean name_is_modified = false;
    private boolean name_is_initialized = false;
    
    private String instrume;
    private boolean instrume_is_modified = false;
    private boolean instrume_is_initialized = false;
    
    private boolean _isNew = true;
    
    /**
     * Do not use this constructor directly, please use the factory method
     * available in the associated manager.
     */
    MetaTable()
    {
    }
    
    /**
     * Getter method for name.
     * <br>
     * Meta Data Information (in progress):
     * <ul>
     * <li>full name: obs_tables.name
     * <li>column size: 30
     * <li>jdbc type returned by the driver: Types.VARCHAR
     * </ul>
     *
     * @return the value of name
     */
    public String getName()
    {
        return name; 
    }

    /**
     * Setter method for name.
     * <br>
     * The new value is set only if compareTo() says it is different,
     * or if one of either the new value or the current value is null.
     * In case the new value is different, it is set and the field is marked as 'modified'.
     *
     * @param newVal the new value to be assigned to name
     */
    public void setName(String newVal) {
        if ((newVal != null && this.name != null && (newVal.compareTo(this.name) == 0)) || 
            (newVal == null && this.name == null && name_is_initialized)) {
            return; 
        } 
        this.name = newVal; 
        name_is_modified = true; 
        name_is_initialized = true; 
    }

    /**
     * Determines if the name has been modified.
     *
     * @return true if the field has been modified, false if the field has not been modified
     */
    public boolean isNameModified() {
        return name_is_modified; 
    }

    /**
     * Determines if the name has been initialized.
     * <br>
     * It is useful to determine if a field is null on purpose or just because it has not been initialized.
     *
     * @return true if the field has been initialized, false otherwise
     */
    public boolean isNameInitialized() {
        return name_is_initialized; 
    }

    /**
     * Getter method for instrume.
     * <br>
     * Meta Data Information (in progress):
     * <ul>
     * <li>full name: obs_tables.instrume
     * <li>column size: 20
     * <li>jdbc type returned by the driver: Types.VARCHAR
     * </ul>
     *
     * @return the value of instrume
     */
    public String getInstrume()
    {
        return instrume; 
    }

    /**
     * Setter method for instrume.
     * <br>
     * The new value is set only if compareTo() says it is different,
     * or if one of either the new value or the current value is null.
     * In case the new value is different, it is set and the field is marked as 'modified'.
     *
     * @param newVal the new value to be assigned to instrume
     */
    public void setInstrume(String newVal) {
        if ((newVal != null && this.instrume != null && (newVal.compareTo(this.instrume) == 0)) || 
            (newVal == null && this.instrume == null && instrume_is_initialized)) {
            return; 
        } 
        this.instrume = newVal; 
        instrume_is_modified = true; 
        instrume_is_initialized = true; 
    }

    /**
     * Determines if the instrume has been modified.
     *
     * @return true if the field has been modified, false if the field has not been modified
     */
    public boolean isInstrumeModified() {
        return instrume_is_modified; 
    }

    /**
     * Determines if the instrume has been initialized.
     * <br>
     * It is useful to determine if a field is null on purpose or just because it has not been initialized.
     *
     * @return true if the field has been initialized, false otherwise
     */
    public boolean isInstrumeInitialized() {
        return instrume_is_initialized; 
    }

    /**
     * Determines if the current object is new.
     *
     * @return true if the current object is new, false if the object is not new
     */
    public boolean isNew() {
        return _isNew;
    }

    /**
     * Specifies to the object if it has been set as new.
     *
     * @param isNew the boolean value to be assigned to the isNew field
     */
    public void isNew(boolean isNew) {
        this._isNew = isNew;
    }

    /**
     * Determines if the object has been modified since the last time this method was called.
     * <br>
     * We can also determine if this object has ever been modified since its creation.
     *
     * @return true if the object has been modified, false if the object has not been modified
     */
    public boolean isModified() {
        return name_is_modified || 
		instrume_is_modified;
    }

    /**
     * Resets the object modification status to 'not modified'.
     */
    public void resetIsModified() {
        name_is_modified = false;
        instrume_is_modified = false;
    }

    /**
     * Copies the passed bean into the current bean.
     *
     * @param bean the bean to copy into the current bean
     */
    public void copy(MetaTable bean) {
        setName(bean.getName());
        setInstrume(bean.getInstrume());
    }

    /**
     * Returns the object string representation.
     *
     * @return the object as a string
     */
    public String toString() {
        return   "\n[obs_tables] "
                 + "\n - obs_tables.name = " + (name_is_initialized ? ("[" + (name == null ? null : name.toString()) + "]") : "not initialized") + ""
                 + "\n - obs_tables.instrume = " + (instrume_is_initialized ? ("[" + (instrume == null ? null : instrume.toString()) + "]") : "not initialized") + ""
            ;
    }

}
