package org.eso.oca.fitsmapper;

import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Vector;

import org.eso.oca.data.JdbcConnectionException;
import org.eso.oca.data.MetaColumn;
import org.eso.oca.data.MetaTable;
import org.eso.oca.data.ObsColumnsDAO;
import org.eso.oca.data.ObsTablesDAO;

/**
 * Class DBFitsMapper
 *
 * Implements the DBFitsMapper for the raw database
 *
 * @version $Revision: 1.3 $
 * @author pnunes, Mar 30, 2005
 */

// $Id: RawDbFitsMapper.java,v 1.3 2005/06/06 10:31:21 szampier Exp $ 
public class RawDbFitsMapper extends DbFitsMapper {
    
    
    /**
     * Singleton
     */
    private static RawDbFitsMapper _instance; 

    
    // For lazy initialization 
    public static synchronized RawDbFitsMapper getInstance() throws SQLException, JdbcConnectionException { 
       if (_instance==null) { 
          _instance = new RawDbFitsMapper(); 
         } 
         return _instance; 
    }    
            
    /**
     * @throws JdbcConnectionException
     * @throws SQLException
     * 
     */
    private RawDbFitsMapper() throws SQLException, JdbcConnectionException {        
        load();
        
    }
    
    
    /**
     * Loads the configuration from the database
     * 
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    private void load() throws SQLException, JdbcConnectionException {                
        // load the obsTables and store them in cache        
        ObsTablesDAO ot = ObsTablesDAO.getInstance();
        tables = new Hashtable();
        
        // iterate obs_tables
        MetaTable [] metaTables = ot.loadAll();               
        for (int i = 0; i < metaTables.length; i++) {
            MetaTable metaTable = metaTables[i];
            
            // insert in the hashtable
            // check if it is already there
            Vector tabs;
            
            String instrum = metaTable.getInstrume();
            
            // if the hashtable already contains this instrument, 
            // get the vector, otherwise create a new one            
            if ( tables.containsKey(instrum )) {
                tabs = (Vector)tables.get(instrum);                
                
            } else {
                tabs = new Vector();
            }
            
            //  add the new ObsTable object to the vector and put it in the hashtable
            tabs.add(metaTable);
            tables.put( instrum , tabs );              
        }
                
        // load the obsColumns and store them in cache        
        ObsColumnsDAO oc = ObsColumnsDAO.getInstance();
        columnsFromKW = new Hashtable();
        columnsFromColumnName = new Hashtable();
        
        
        // iterate obscolumns
        MetaColumn [] metaColumns = oc.loadAll();               
        for (int i = 0; i < metaColumns.length; i++) {
            MetaColumn metaColumn = metaColumns[i];
            
            // insert in the hashtables
            //System.out.println(obsColumn.toString());
            columnsFromKW.put( metaColumn.getTname() + metaColumn.getKeyword()  , metaColumn );
            columnsFromColumnName.put ( metaColumn.getTname()+ metaColumn.getName() , metaColumn );
        }                
    }
    
                   
   
    

}





