/* Generated By:JJTree: Do not edit this line. ASTSelectAssociateStatement.java */

package org.eso.oca.parser;

import java.math.BigDecimal;
import java.util.Collections;
import java.util.Comparator;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.oca.fits.OCAFile;

public class ASTSelectAssociateStatement extends SimpleNode {

    final static String CALIB_FILES = "calibFiles";

    final static String INPUT_FILES = "inputFiles";

    final static String RAW_FILES = "rawFiles";

    final static String TIMEDIFF = "TIMEDIFF";

    public String dataSource;

    public String alias;

    public ASTSelectAssociateStatement(int id) {
        super(id);
    }

    public ASTSelectAssociateStatement(OcaParser p, int id) {
        super(p, id);
    }

    /***************************************************************************
     * Accept the visitor.
     * 
     * @throws OcaVisitorException
     */
    public Object jjtAccept(OcaParserVisitor visitor, Object data)
            throws OcaVisitorException {
        return visitor.visit(this, data);
    }

    static Logger logger = Logger.getLogger(ASTSelectAssociateStatement.class);

    private class CalibFileComparator implements Comparator {
        public int compare(Object a, Object b) {
            Object x = ((Hashtable) a).get(TIMEDIFF);
            Object y = ((Hashtable) b).get(TIMEDIFF);
            return (((BigDecimal) x).compareTo((BigDecimal) y));
        }
    }

    public void interpret(OCAState state) throws InterpretationException {
        logger.debug("interpret() called.");

        /* files selected by the association rule */
        Vector selectedFiles = new Vector();

        /* the data set containing files to associate */
        Vector associationDatabase = null;

        if (dataSource == CALIB_FILES)
            associationDatabase = state.globCalibFiles;
        else
            associationDatabase = state.globInputFiles;

        /*
         * use modified MJD-OBS of input (exemplar) file as reference timestamp.
         * If modified MJD-OBS is not available, use normal MJD-OBS, otherwise
         * report an error and use default value 0.0
         */
        BigDecimal inputFileMjdObs = null;
        inputFileMjdObs = (BigDecimal) state.inputFileSymtab.get(OCAFile.MJD_OBS_MOD);
        if (inputFileMjdObs == null) {
            logger.warn("Modified MJD-OBS is not available in exemplar file "
                    + state.inputFileSymtab.get(OCAFile.FILENAME));
            inputFileMjdObs = (BigDecimal) state.inputFileSymtab.get(OCAFile.MJD_OBS);
            if (inputFileMjdObs == null) {
                logger.error("MJD-OBS is not available in exemplar file "
                        + state.inputFileSymtab.get(OCAFile.FILENAME));
                inputFileMjdObs = new BigDecimal(0.0);
            }
        }

        /* loop through calibration files */
        for (Iterator it = associationDatabase.iterator(); it.hasNext();) {
            /* load symtab with current association file
             * before invoking the interpretation of the where clause */
            state.symtab = (Hashtable) it.next();

            /* interpret where clause */
            jjtGetChild(0).interpret(state);
            if (((Boolean) state.stack[state.top--]).booleanValue()) {
                /* where clause is true => select current file */
                /* make a copy of the current file ! */
                Hashtable selectedFile = new Hashtable(state.symtab);
                
                selectedFile.put(OCAFile.ALIAS, alias);
                BigDecimal calibFileMjdObs = null;
                Object tmpMjdObs = selectedFile.get(OCAFile.MJD_OBS);
                if (tmpMjdObs instanceof BigDecimal) {
                    calibFileMjdObs = (BigDecimal) tmpMjdObs;
                } else {
                    calibFileMjdObs = new BigDecimal(0.0);
                    logger.debug("Null MJD-OBS in calib file "
                            + selectedFile.get(OCAFile.FILENAME));
                }

                /*
                 * add the metakeyword TIMEDIFF to the selected file,
                 * containing the time difference (in days) to the input file.
                 */
                logger.debug("inputFileMjdObs=" + inputFileMjdObs
                        + " calibFileMjdObs=" + calibFileMjdObs);
                selectedFile.put(TIMEDIFF, calibFileMjdObs.subtract(inputFileMjdObs)
                        .abs());
                selectedFiles.add(selectedFile);
            }
        }
        
        /*
         * check if the cardinality of the result matches the requested minRet
         * and maxRet
         */
        Integer retCount = new Integer(selectedFiles.size());
        if (retCount.compareTo(state.minRet) < 0) {
            /* not enough associations found */
            StringBuffer msg = new StringBuffer();
            msg.append("Could not find enough products of type " + alias);
            msg.append(" for " + state.inputFileSymtab.get(OCAFile.FILENAME) + ".");
            msg.append(" Requested " + state.minRet + ". Found " + retCount + ".");
            logger.warn(msg);
            state.isABValid = false;
        } else if (retCount.compareTo(state.maxRet) > 0) {
            /* too many associations found. Only the maxtRet closest in time
             * will be returned.
             */
            StringBuffer msg = new StringBuffer();
            msg.append("Discarding "
                    + (retCount.intValue() - state.maxRet.intValue())
                    + " surplus products of type " + alias + " for "
                    + state.inputFileSymtab.get(OCAFile.FILENAME) + ".");
            logger.debug(msg);
            /* sort by TIMEDIFF */
            Collections.sort(selectedFiles, new CalibFileComparator());
            /* discard surplus calibrations. keep only the closest in time */
            selectedFiles.setSize(state.maxRet.intValue());
        }
        /* add the files selected by this rule to the pool
         * of associated files */
        state.associatedFiles.addAll(selectedFiles);
        /* set the cardinality to its default value */
        state.minRet = new Integer(1);
        state.maxRet = new Integer(1);
    }
}