package org.eso.util.dal;

import java.sql.Connection;
import java.sql.SQLException;

import org.apache.log4j.Logger;

/**
 * Service used to ingest a file entry in ngas_files table and update the correspondent entry in ngas_disks
 * 
 * @author Vincenzo Forchi` <vforchi@eso.org>
 *
 */
public class FileIngestionService {
	
	/** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(FileIngestionService.class);
 
    private static final String thisClass = "FileIngestionService";
    
	private String d_dbUrl			= null;
    private String d_dbUsername		= null;
    private String d_dbPassword		= null;
    private String d_dbDiskTable	= null;
    private String d_dbFileTable	= null;
 
    /**
     * Constructor
     * 
     * @param dbUrl	url of the database server
     * @param dbUsername database username
     * @param dbPassword database password
     * @param dbDiskTable ngas_disks table
     * @param dbFileTable ngas_files table
     */
	public FileIngestionService(String dbUrl, String dbUsername, String dbPassword, String dbDiskTable, String dbFileTable) {
		d_dbUrl			= dbUrl;
		d_dbUsername	= dbUsername;
		d_dbPassword	= dbPassword;
		d_dbDiskTable	= dbDiskTable;
		d_dbFileTable	= dbFileTable;
	}
	
	/**
	 * Ingest the given entries in the database
	 * 
	 * @param file the file entry to be inserted
	 * @param disk the disk entry to be updated
	 * @throws DAOException
	 */
	public void ingestIntoDb(NgasFile file, NgasDisk disk) throws DAOException {
		logger.trace(thisClass + "::ingestionIntoDb()");
		
		Connection conn = null;
		try {
			conn = SybaseConnectionService.getConnection(d_dbUrl, d_dbUsername, d_dbPassword);
				
			//logger.debug("Creating disk dao syb");
			//NgasDiskDAOSyb diskDAO = new NgasDiskDAOSyb(conn, d_dbDiskTable, d_dbFileTable);
			logger.debug("Creating file dao syb");
			NgasFileDAOSyb fileDAO = new NgasFileDAOSyb(conn, d_dbFileTable);
			//NgasFileDAOSyb fileDAO = new NgasFileDAOSyb(d_dbUrl, d_dbUsername, d_dbPassword, d_dbFileTable);
		
			conn.setAutoCommit(false);
		
			//System.out.println(file.toString());
			logger.debug("ingesting file " + file.getFileName());
			fileDAO.save(file);
			//throw new DAOException("File [" + file.getFileName() + "] is already in the database");
			//logger.debug("updating " + disk.getLogicalName() + ": " + disk.getNumberOfFiles() + "files, " + disk.getBytesStored() + "bytes stored");
			//diskDAO.update(disk);
			//logger.debug("updated " + disk.getLogicalName());
			conn.commit();
			logger.debug("Transaction committed");
			//SybaseConnectionService.closeConnection(conn);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (DAOException e) {
			try {
				conn.rollback();
				String message = "Error while ingesting file " + file.getFileId() + " on disk " + disk.getDiskId() + ": " + e.getMessage();
				throw new DAOException(message);
			} catch (SQLException ex) {
				throw new DAOException(ex.getMessage());
			}
		} finally {
			if (conn != null)
			{
				try {
					conn.setAutoCommit(true);
					SybaseConnectionService.closeConnection(conn);
				} catch (ConnectionServiceException ex) {
					throw new DAOException(ex.getMessage());
				} catch (SQLException ex) {
					throw new DAOException(ex.getMessage());
				}
			}
		}
	}
}
