package org.eso.util.dal;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;


/**
 * Implementation of the NgasFileDAO for Sybase.
 * 
 * @author mpadovan
 *
 */
public class NgasFileDAOSyb implements NgasFileDAO {

	/** Apache Log4J logger for this class namespace. */
	private static final Logger logger = Logger.getLogger(NgasFileDAOSyb.class);
	
	private final String dbUrl;
    private final String dbUsername;
    private final String dbPassword;
    private final String dbTable;
    
    private final Connection dbConnection;
    
	public NgasFileDAOSyb(String dbUrl, String dbUsername, String dbPassword, String dbTable) {
		this.dbUrl		= dbUrl;
		this.dbUsername	= dbUsername;
		this.dbPassword	= dbPassword;
		this.dbTable    = dbTable;
		
		dbConnection = null;
	}
	
	public NgasFileDAOSyb(Connection dbConnection, String dbTable) {
		this.dbUrl		= "";
		this.dbUsername	= "";
		this.dbPassword	= "";

		this.dbConnection = dbConnection;
		this.dbTable = dbTable;
	}
	
	public int delete(NgasFile ngasFile) throws DAOException {
		// TODO
		throw new DAOException("Not yet implemented.");
	}
	
	public int deleteByDiskId(String diskId) throws DAOException {
		final String sqlStatement = "delete from " + dbTable + " where disk_id='" + diskId + "'";
		try {
			return this.executeUpdateStatement(sqlStatement);
		} catch (Exception e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eso.util.dal.NgasFileDAO#deleteByDiskIdAndFileId(java.lang.String,java.lang.String)
	 */
	public int deleteByDiskIdAndFileId(String diskId, String fileId)
		throws DAOException {
		
		final String sqlStatement = "delete from " + dbTable + " where disk_id='" + diskId + "' and file_id='" + fileId + "'";
		try {
			return this.executeUpdateStatement(sqlStatement);
		} catch (Exception e) {
			throw new DAOException(e.getMessage());
		}
	}

	public List<NgasFile> findByFileId(String fileId)
		throws DAOException {
		
		return find(fileId, null, null);
	}
		

	public List<NgasFile> find(String fileId, String diskId, Integer version)
			throws DAOException {
		
		final String logHead = "NgasFileDAOSyb::find() - ";
		
		final String sqlStatement = getFindSQLStatement(fileId, diskId, version);
		
		List<NgasFile> ngasFileList = null;
		Connection connection = null;
		
		try {
			connection = this.getConnection();
			ResultSet rs = SybaseConnectionService.executeStatement(connection, sqlStatement);
			ngasFileList = NgasFileDAOSyb.toNgasFileList(rs);
		} catch (Exception e) {
			throw new DAOException(e.getMessage());
		} finally {
			try {
				SybaseConnectionService.closeConnection(connection);
			} catch (ConnectionServiceException e) {
				// just log an error
				logger.error(logHead + "Cannot close connection.");
			}
		}

		return ngasFileList;
	}

	
	public List<NgasFile> findAll() throws DAOException {
		// TODO
		throw new DAOException("Not yet implemented.");
	}
	
	
	public void save(NgasFile ngasFile)
		throws DAOException {
		
		final String sqlStatement = getSaveSQLStatement(ngasFile);
		
		try {
			this.executeStatement(sqlStatement);
		} catch (Exception e) {
			throw new DAOException(e.getMessage());
		}
	}

	
	public int update(NgasFile ngasFile) throws DAOException {
		// TODO
		throw new DAOException("Not yet implemented.");
	}
	

	public boolean saveIfNotFound(NgasFile ngasFile) throws DAOException {
		
		final String logHead = "NgasFileDAOSyb::saveIfNotFound() - ";

		boolean alreadyThere = false;
		
		Connection connection = null;

		// the following SQL statements are executed as a single transaction
		
		try {

			connection = this.getConnection();

			connection.setAutoCommit(false);
			
			ResultSet rs = SybaseConnectionService.executeStatement(
					connection,
					getFindSQLStatement(
							ngasFile.getFileId(),
							ngasFile.getDiskId(),
							ngasFile.getFileVersion()
							)
					);

			// this is ugly, but apparently it is the only safe way to do it
			int rowCount = 0;
			while(rs.next()) ++rowCount;

			alreadyThere = rowCount > 0;

			// insert the ngas file if necessary
			if(!alreadyThere) {
				SybaseConnectionService.executeStatement(
						connection,
						getSaveSQLStatement(ngasFile)
						);
			}
			
		} catch (IllegalArgumentException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		}

		// do the commit
		try {
			connection.commit();
		} catch (SQLException e) {
			throw new DAOException("It was not possible to complete the transaction. Reason: " + e.getMessage());
		} finally {
			try {
				connection.setAutoCommit(true);
				connection.close();
			} catch (SQLException e) {
				// just log an error
				logger.error(logHead + "Cannot close connection.");
			}
		}

		return alreadyThere;
	}

	
	/**
	 * Creates a list of NgasFile instances out of a result set. For each
	 * row of the result set an NgasFile instance is added to the list.
	 * 
	 * @param rs The result set.
	 * @return The list of NgasFile instances.
	 * @throws DAOException in case of error.
	 */
	public static List<NgasFile> toNgasFileList(final ResultSet rs)
		throws DAOException {
		
		final List<NgasFile> ngasFileList = new ArrayList<NgasFile>();

		try {
			
			while (rs.next()) {
				
				NgasFile ngasFile = new NgasFile();

				ngasFile.setDiskId(rs.getString("disk_id"));
				ngasFile.setFileName(rs.getString("file_name"));
				ngasFile.setFileId(rs.getString("file_id"));
				ngasFile.setFormat(rs.getString("format"));
				ngasFile.setFileSize(rs.getLong("file_size"));
				ngasFile.setUncompressedFileSize(rs.getLong("uncompressed_file_size"));
				ngasFile.setCompression(rs.getString("compression"));
				ngasFile.setIngestionDate(rs.getDate("ingestion_date"));
				ngasFile.setIgnore(rs.getInt("ignore"));
				ngasFile.setChecksum(rs.getString("checksum")); 
				ngasFile.setChecksumPlugin(rs.getString("checksum_plugin")); 
				ngasFile.setFileVersion(rs.getInt("file_version"));
				ngasFile.setFileStatus(rs.getString("file_status"));
				ngasFile.setCreationDate(rs.getDate("creation_date"));

				ngasFileList.add(ngasFile);
			}
			
		} catch (SQLException e) {
			throw new DAOException("Error while parsing result set. Reason: " + e.getMessage());
		}
		
		return ngasFileList;
	}

	
	private String fieldNamesToString(NgasFile ngasFile) {
		return	"disk_id," +
				"file_name," +
				"file_id," +
				"format," +
				"file_size," +
				"uncompressed_file_size" +
				(ngasFile.getCompression() == null ? "" : ",compression") +
				",ingestion_date" +
				(ngasFile.getIgnore() == null ? "" : ",ignore") +
				(ngasFile.getChecksum() == null ? "" : ",checksum") +
				(ngasFile.getChecksumPlugin() == null ? "" : ",checksum_plugin") +
				(ngasFile.getFileVersion() == null ? "" : ",file_version") +
				(ngasFile.getFileStatus() == null ? "" : ",file_status") +
				(ngasFile.getCreationDate() == null ? "" : ",creation_date");
	}
	
	
	private String valuesToString(NgasFile ngasFile) {
		return	"'" + ngasFile.getDiskId() + "'," +
				"'" + ngasFile.getFileName() + "'," +
				"'" + ngasFile.getFileId() + "'," +
				"'" + ngasFile.getFormat() + "'," +
				ngasFile.getFileSize() + "," +
				ngasFile.getUncompressedFileSize() +
				(ngasFile.getCompression() == null ? "" : ",'" + ngasFile.getCompression() + "'") +
				",'" + SybaseConnectionService.dateFormat.format(ngasFile.getIngestionDate()) + "'" +
				(ngasFile.getIgnore() == null ? "" : "," + ngasFile.getIgnore()) +
				(ngasFile.getChecksum() == null ? "" : ",'" + ngasFile.getChecksum() + "'") +
				(ngasFile.getChecksumPlugin() == null ? "" : ",'" + ngasFile.getChecksumPlugin() + "'") +
				(ngasFile.getFileVersion() == null ? "" : "," + ngasFile.getFileVersion()) +
				(ngasFile.getFileStatus() == null ? "" : ",'" + ngasFile.getFileStatus() + "'") +
				(ngasFile.getCreationDate() == null ? "" : ",'" + SybaseConnectionService.dateFormat.format(ngasFile.getCreationDate()) + "'");
	}
	
	
	/**
	 * Returns the SQL statement to insert the ngas file in the database.
	 */
	private String getSaveSQLStatement(NgasFile ngasFile) {
		return	"insert into " + dbTable + "(" + fieldNamesToString(ngasFile) + ") values (" + valuesToString(ngasFile) + ")";
	}
	
	
	/**
	 * Returns the SQL statement to find the ngas file with the specified filed ID, disk ID, and version number.
	 */
	private String getFindSQLStatement(String fileId, String diskId, Integer version)
		throws DAOException {
		
		if(fileId == null) {
			throw new DAOException("File ID cannot be null.");
		}
		
		return	"select * from " + dbTable +
				" where " +
				"file_id='" + fileId + "'" +
				(diskId != null ? (" and disk_id='" + diskId + "'") : "") +
				(version != null ? (" and file_version=" + version) : "");
	}

	
	private Connection getConnection()
		throws IllegalArgumentException, ConnectionServiceException {
		
		if(this.dbConnection == null) {
			return SybaseConnectionService.getConnection(dbUrl, dbUsername, dbPassword);
		} else {
			return this.dbConnection;
		}
	}
	
    private void executeStatement(String sqlStatement)
    	throws IllegalArgumentException, ConnectionServiceException {
    	
    	if(this.dbConnection == null) {
    		// get a new connection, execute the query, and close the connection
    		final Connection connection = SybaseConnectionService.getConnection(dbUrl, dbUsername, dbPassword);
			SybaseConnectionService.executeStatement(connection, sqlStatement);
			SybaseConnectionService.closeConnection(connection);
    	} else {
    		// use the connection passed on the constructor, but do not close it !
			SybaseConnectionService.executeStatement(this.dbConnection, sqlStatement);
    	}
    }

    private int executeUpdateStatement(String sqlStatement)
		throws IllegalArgumentException, ConnectionServiceException {
		
    	int affectedRows = 0;
    	
		if(this.dbConnection == null) {
			// get a new connection, execute the query, and close the connection
			final Connection connection = SybaseConnectionService.getConnection(dbUrl, dbUsername, dbPassword);
			affectedRows = SybaseConnectionService.executeUpdateStatement(connection, sqlStatement);
			SybaseConnectionService.closeConnection(connection);
		} else {
			// use the connection passed on the constructor, but do not close it !
			affectedRows = SybaseConnectionService.executeUpdateStatement(this.dbConnection, sqlStatement);
		}
		
		return affectedRows;
	}

    public Long getSize(List<String> fileIds) throws DAOException {
    	/* return value */
    	Long res = 0L;

    	/**
    	 * 1-100   => 1
    	 * 101-200 => 2
    	 * 201-300 => 3
    	 */
    	int numberOfIterations = (fileIds.size()+99)/100; 
    	
    	for (int i = 0; i < numberOfIterations; i++) {
    		int start = i * 100;
    		int stop = Math.min((i+1)*100, fileIds.size());
    		res += getSize(fileIds, start, stop);
    	}

    	return res;
    }
    
    private Long getSize(List<String> fileIds, int startIdx, int stopIdx) throws DAOException {
    	
    	final String logHead = "NgasFileDAOSyb::getSize() - ";

		Long result = 0L;
		
    	Connection connection = null;

		try {

			connection = this.getConnection();

			String query = "select distinct file_id,file_size from " + this.dbTable + " where file_id in (";
			for (int i = startIdx; i < stopIdx; i++) {
				query += "'" + fileIds.get(i) + "',";
			}
			query = query.substring(0, query.length()-1);
			query += ")";
			
			ResultSet rs = SybaseConnectionService.executeStatement(connection,query);
			
			while (rs.next()) {
				result += rs.getLong("file_size");
			}

		} catch (IllegalArgumentException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} finally {
			try {
				connection.close();
			} catch (SQLException e) {
				// just log an error
				logger.error(logHead + "Cannot close connection.");
			}
		}
		return result;
    }    
}
