package org.eso.util.dal;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.List;
import java.util.ArrayList;

import org.apache.log4j.Logger;

public class NgasFileSafeDAOSyb implements NgasFileSafeDAO {

	/** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(NgasFileSafeDAOSyb.class);
    
    /** database's connection parameters */
    private String d_dbUrl			= null;
    private String d_dbUsername		= null;
    private String d_dbPassword		= null;
    
    /** ngas_file_safe table */
    private String d_dbTable		= null;
    
    private Connection d_connection	= null;
	
	public NgasFileSafeDAOSyb(String dbUrl, String dbUsername, String dbPassword, String dbTable) {
		d_dbUrl			= dbUrl;
		d_dbUsername	= dbUsername;
		d_dbPassword	= dbPassword;
		d_dbTable		= dbTable;
	}
	
	public NgasFileSafeDAOSyb(Connection connection, String dbTable) {
		d_connection	= connection;
		d_dbTable		= dbTable;
	}
    
	public void delete(NgasFileSafe ngasFile) throws DAOException {
		throw new DAOException("Method not implemented yet");
	}

	public List<NgasFileSafe> findAll() throws DAOException {
		throw new DAOException("Method not implemented yet");
	}

	public List<NgasFileSafe> findByFileId(String fileId) throws DAOException {
		return findByFileIdAndVersion(fileId, 1);
	}

	public List<NgasFileSafe> findByFileIdAndVersion(String fileId,
			Integer version) throws DAOException {
		String query = "select * from " + d_dbTable + " where file_id='" + fileId + "' and file_version=" + version;
		return executeQuery(query);
	}
	
	public List<NgasFileSafe> findByFileIdIgnoreVersion(String fileId)
		throws DAOException {
		
		String query = "select * from " + d_dbTable + " where file_id='" + fileId + "'";
		return executeQuery(query);
	}

	public void save(NgasFileSafe ngasFile) throws DAOException {
		throw new DAOException("Method not implemented yet");
	}

	public boolean saveIfNotFound(NgasFileSafe ngasFile) throws DAOException {
		throw new DAOException("Method not implemented yet");
	}

	public void update(NgasFileSafe ngasFile) throws DAOException {
		throw new DAOException("Method not implemented yet");
	}
	
	/**
	 * Convenience method used for database access
	 * @param query
	 * @return a List of NgasFileSafe objects
	 * @throws DAOException
	 */
	private List<NgasFileSafe> executeQuery(String query) throws DAOException {
		try {
			List<NgasFileSafe> res = null;
			if (d_connection == null)
			{
				Connection conn = SybaseConnectionService.getConnection(d_dbUrl, d_dbUsername, d_dbPassword); 
				res = toNgasFileSafeList(SybaseConnectionService.executeStatement(conn, query));
				SybaseConnectionService.closeConnection(conn);
			}
			else
				res = toNgasFileSafeList(SybaseConnectionService.executeStatement(d_connection, query));
			return res;
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}

	/**
	 * Converts the result of a database query to a list of NgasDisk objects
	 * 
	 * @param rs the result of the query
	 * @return a list of NgasDisk objects
	 * @throws DAOException
	 */
	public static List<NgasFileSafe> toNgasFileSafeList(ResultSet rs) throws DAOException {
		List<NgasFileSafe> res = new ArrayList<NgasFileSafe>();
		try {
			while (rs.next()) {
				NgasFileSafe file = new NgasFileSafe();

				file.setFileId(rs.getString("file_id"));
				file.setIngestionDate(rs.getDate("ingestion_date"));
				file.setFileVersion(rs.getInt("file_version"));
				
				res.add(file);
			}
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (NullPointerException e) {}		
		return res;
	}

	public static void main(String[] args) {
		NgasFileSafeDAO dao = new NgasFileSafeDAOSyb("acdbdev.hq.eso.org:6789", "ngas", "ngas_pw", "ngastst1..ngas_files_safe");
		try {
			for (NgasFileSafe f: dao.findByFileIdAndVersion("prova",2))
				System.out.println(f.toString());
			for (NgasFileSafe f: dao.findByFileId("prova"))
				System.out.println(f.toString());
		} catch (DAOException e) {
			System.out.println(e.getMessage());
		}
	}
}
