package org.eso.util.dal;

import java.util.List;

/**
 * DAO interface for ReplicationRequest objects
 * 
 * @author Vincenzo Forchi` <vforchi@eso.org>
 *
 */
public interface ReplicationRequestDAO {
	
	/**
	 * Find all the RR with a given status
	 * 
	 * @param status
	 * @return a list of RR
	 * @throws DAOException
	 */
	public List<ReplicationRequest> findByStatus(ReplicationRequest.Status status) throws DAOException;
	
	/**
	 * Update a RR
	 * 
	 * @param request the RR whose status is to be updated
	 * @throws DAOException
	 */
	public void update(ReplicationRequest request) throws DAOException;
	
	/**
	 * Insert a new RR
	 * 
	 * @param request the new RR
	 * @throws DAOException
	 */
	public void insert(ReplicationRequest request) throws DAOException;
	
	/**
	 * Deletes an entry 
	 * 
	 * @param request the RR to be deleted
	 * @throws DAOException
	 */
	public void delete(ReplicationRequest request) throws DAOException;
	
}
