package org.eso.util.datatransfer;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import org.apache.commons.dbcp.BasicDataSource;


import org.apache.log4j.Logger;



/**
 * This class provides a simple user-friendly interface to connect to a
 * database through JDBC.
 */
public class SybaseConnectionService {
	
    /** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(SybaseConnectionService.class);

    private BasicDataSource dataSource;
	
	/**
	 * Constructor.
	 * @param serverName
	 * @param userName
	 * @param userPassword
	 */
	public SybaseConnectionService(String serverUrl, String userName, String userPassword) {

		dataSource = new BasicDataSource();
		dataSource.setDriverClassName("com.sybase.jdbc2.jdbc.SybDriver");
		dataSource.setUsername(userName);
		dataSource.setPassword(userPassword);
		dataSource.setUrl("jdbc:sybase:Tds:" + serverUrl);
	}
	
	/**
	 * Constructor.
	 * @param databaseAlias
	 * @throws Exception
	 */
	public SybaseConnectionService(String databaseAlias) throws Exception {
		throw new Exception("ArchiveService::ArchiveService(String databaseAlias) - to be implemented.");
		// TODO
	}

	/**
	 * Connects to the database and executes the query.
	 * @param query The query to execute.
	 * @return true if successful, otherwise false.
	 */
	public boolean executeQuery(String query) {
		return executeQuery(query, null);
	}

	/**
	 * Connects to the database and executes the query.
	 * @param query The query to execute.
	 * @param resultSet The object to be used to return the query result set, if any.
	 * @return true if successful, otherwise false.
	 */
	public boolean executeQuery(String query, ResultSet[] resultSet) {
		
		boolean result = false;
		
		String logHead = "ArchiveService::executeQuery() - ";
		
		Connection connection = null;
		
		try {
			
			connection = dataSource.getConnection();

			logger.debug(logHead + "Acquired connection to the database.");
			
			Statement statement = connection.createStatement();
			
			if( statement.execute(query) && resultSet != null ) {
				
				// the query generated a result set and the caller needs it
				//
				// note: if execute() returns true, it means that a result set
				//       is available
				
				resultSet[0] = statement.getResultSet();
			}

			logger.debug(logHead + "Query successfully executed (" + query + ").");
			
			result = true;
			
		} catch (SQLException e) {
			
			logger.error(logHead + "Error while executing query. Reason: " + e.getMessage());
			
		} finally {
			
			try {
				if(connection != null) {
					connection.close();
					logger.debug(logHead + "Released connection to the database.");
				}
			} catch(Exception e) {
				logger.error(logHead + "Cannot release connection to the database. Reason: " + e.getMessage());
			}
			
		}
		
		return result;
	}
}
