package org.eso.util.filesystem; 

import java.io.File; 

import org.apache.log4j.Logger; 

/** Instances of this class are used with the 
 * {@link FileSystemOperations#checkForDirectories()} method. Each instance 
 * defines a single directory to check for. 
 * 
 * @author Neil Ferguson ESO/SDD/DFI/BET <nferguso@eso.org> 
 */
public class Directory 
{
    /** Apache Log4J logger. */ 
    static final Logger logger = Logger.getLogger(Directory.class);

    /** The name of this class for logging purposes. */ 
    private static final String classLogName = "Directory"; 

    /** The full path to the directory. */
    final File path; 

    /** A text snippet for use in log messages of the form "...the 
     * XXX directory..." where XXX will be substituted with the given 
     * text. Set this to null if it is not required. 
     */ 
    final String logText; 

    /** True if the directory is required. */ 
    final boolean isRequired; 

    /** True if the directory should be created when it is required, 
     * but missing. 
     */ 
    final boolean createDir; 

    /** Constructs a new Directory. All parameters are immutable after 
     * construction. 
     * 
     * @param path the full path to the directory. 
     * @param logText see {@link #logText}. 
     * @param isRequired true if the directory is required. 
     * @param createDir true if the directory should be created when it is 
     *                  required, but missing. 
     * 
     * @throws NullPointerException if path is null. 
     */
    public Directory(final File path, 
                     final String logText, 
                     final boolean isRequired, 
                     final boolean createDir) throws NullPointerException 
    {
        final String methodLogName = classLogName + "::Directory()"; 

        logger.trace(methodLogName); 

        if(path == null) 
        {
            final String message = 
                methodLogName + " - path must not be null."; 

            logger.fatal(message); 
            throw new NullPointerException(message); 
        }

        this.path = path; 
        this.logText = logText; 
        this.isRequired = isRequired; 
        this.createDir = createDir; 
    }
}
