package org.eso.util.requesthandler;

/** Interface for dao classes dealing with data request options */
public interface DatasetRequestOptionDAO {

    /**
     * Retrieves option meta data for a given archive and a given medium. Only
     * one option should be retrieved.
     * 
     * @param mediaName
     *            media to be used : CDR, DVR, MAG, LAN, FTP, AUT
     * @param archive
     *            archive associated to the option : SAF or HST.
     * @return option meta data
     */
    // RIM - even here I don't like the talk about bits. It's so C. Better to
    // pass in a higher-level
    // abstraction and extract the bits at the last moment, in the
    // implementation.
    // RIM - I fully agree. Even with backwards compatibility on DB level, there
    // must be a way
    // to handle this on a more abstract level. Bitmasks at interface level are
    // soo ugly.
    // Also, I don't understand what the incoming reqOption means
    // REPLY : use of bit mask removed
    DatasetRequestOption findByMediumAndArchive(
            DatasetRequestOptionName mediaName, ArchiveName archive)
            throws DAOException;

    /**
     * Retrieves option meta data for a given option name and archive. Only one
     * option should be retrieved.
     * 
     * @param optionName
     *            name of the option to be retrieved : CDR, DVR, MAG, LAN, FTP,
     *            AUT, VOSTORE, CALSEL, GZIP, GUNZIP, VIMREP, WFIPREVI, PREIMA,
     *            ESOFIXHD, BEOWFIPR, ORIGNAME
     * @param archive
     *            archive associated to the option : SAF or HST.
     * @return option meta data
     * 
     * @throws DAOException
     */
    DatasetRequestOption findByOptionNameAndArchive(
            DatasetRequestOptionName optionName, ArchiveName archive)
            throws DAOException;

}
