package org.eso.util.requesthandler;

import java.io.Serializable;

/**
 * Implementation of a composite identifier of a data set request option. It
 * will be made of the identifier of the archive (SAF or HST) the option belongs
 * to and of a bit mask
 * 
 * @author cmoins
 * 
 */
public class DatasetRequestOptionId implements Serializable {

    /**
     * Number of bytes of a request option bit mask.
     */
    public static final Integer OPTION_BIT_MASK_BYTES_NUMBER = 6;

    private static final long serialVersionUID = 1L;
    // option mask
    private byte[] reqOption;
    // archive to which this option is associated
    private ArchiveName archive;

    public DatasetRequestOptionId() {
    }

    public DatasetRequestOptionId(byte[] reqOption, ArchiveName archive) {
        this.reqOption = reqOption;
        this.archive = archive;
    }

    public byte[] getReqOption() {
        return reqOption;
    }

    public void setReqOption(byte[] reqOption) {
        if (reqOption.length <= OPTION_BIT_MASK_BYTES_NUMBER) {
            this.reqOption = reqOption;
        }
    }

    public ArchiveName getArchive() {
        return archive;
    }

    public void setArchive(ArchiveName archive) {
        this.archive = archive;
    }

    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null)
            return false;
        if (!(o instanceof DatasetRequestOptionId))
            return false;
        final DatasetRequestOptionId dsReqOptId = (DatasetRequestOptionId) o;
        if (!reqOption.equals(dsReqOptId.getReqOption()))
            return false;
        if (!archive.equals(dsReqOptId.getArchive()))
            return false;
        return true;
    }

    public int hashCode() {
        return (archive.hashCode() + reqOption.hashCode());
    }
}
