package org.eso.util.dal.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.TimeZone;

import org.eso.util.dal.PackageAuxFilesDAOSyb;
import org.eso.util.dal.SybaseConnectionService;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class PackageAuxFilesDAOSybTest {
	
	private static PackageAuxFilesDAOSyb dao = null;
	private static Statement stmt = null;
	
	private static String DBTABLE = "rhandler..pi_pkg_aux_files";
	
	private static final Integer pkgId = 44444444;
	
	private static Map<String, List<String>> testFiles;
	private static List<String> testFileIds;
	
	private static int MAX_FILES = 500;
	private static int AUXFILES_PER_FILE = 5;
	
	private static Connection conn1;
	private static Connection conn2;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		TimeZone.setDefault(TimeZone.getTimeZone("UTC"));
		conn1 = SybaseConnectionService.getConnection("acdbdev:6789", "pipack", "pipack_ps");
		dao = new PackageAuxFilesDAOSyb(conn1, DBTABLE);
		conn2 = SybaseConnectionService.getConnection("acdbdev:6789", "pipack", "pipack_ps");
		stmt = conn2.createStatement();
		
		/* delete previous entries */
		truncateTable();
		String query = "delete from rhandler..pi_pkg_files where pkg_id=" + pkgId;
		stmt.execute(query);
		query = "delete from rhandler..pi_pkgs where pkg_id=" + pkgId;
		stmt.execute(query);
		
		/* insert needed entries in other tables */
		query = "insert into rhandler..pi_pkgs values (" + pkgId + ", 'TESTRUN', 'TESTINSTRU', 'OPEN', 'TESTDATA')";
		stmt.execute(query);
		query = "insert into rhandler..pi_pkg_files values (" + pkgId + ", 'TESTFILENAME0', 'FILEID0', 'TESTPATH0', 'R', '2000-01-01', '2000-01-01')";
		stmt.execute(query);
		query = "insert into rhandler..pi_pkg_files values (" + pkgId + ", 'TESTFILENAME1', 'FILEID1', 'TESTPATH1', 'R', '2000-01-01', '2000-01-01')";
		stmt.execute(query);
		
		/* create test files */
		testFiles = new HashMap<String, List<String>>();
		for (int i = 0; i < (MAX_FILES/AUXFILES_PER_FILE); i++) {
			List<String> auxFiles = new ArrayList<String>();
			for (int j = 0; j < AUXFILES_PER_FILE; j++) {
				auxFiles.add("AUXFILE" + j);
			}
			String istr = String.valueOf(i);
			if (i < 10)
				istr = "0" + istr;
				
			testFiles.put("FILEID" + istr, auxFiles);
		}
		
		testFileIds = new ArrayList<String>(testFiles.keySet());
		Collections.sort(testFileIds);
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		truncateTable();
		String query = "delete from rhandler..pi_pkg_files where pkg_id=" + pkgId;
		stmt.execute(query);
		query = "delete from rhandler..pi_pkgs where pkg_id=" + pkgId;
		stmt.execute(query);
		
		conn1.close();
		conn2.close();
	}

	@Before
	public void setUp() throws Exception {		
	}

	@Test
	public void testGet1() {
		try {
			truncateTable();
			insertTestEntry(testFiles, testFileIds.subList(0, 1));
						
			String id = testFileIds.get(0);
			
			/***************************************************************/
			dao.getAuxFiles(id);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageAuxFilesDAOSyb.FILEID_KW + "='" + id + "'"));
			ResultSet rs = stmt.getResultSet();
			for (String auxFile: testFiles.get(id)) {
				assertTrue(rs.next());
				assertEquals((Integer) pkgId, (Integer) rs.getInt(PackageAuxFilesDAOSyb.PACKAGEID_KW));
				assertEquals(auxFile, rs.getString(PackageAuxFilesDAOSyb.AUXFILE_KW));
				assertEquals(id, rs.getString(PackageAuxFilesDAOSyb.FILEID_KW));
			}
			assertFalse(rs.next());
				
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testGet2() {
		try {
			truncateTable();
			
			List<String> ids = testFileIds.subList(0, 10);
			
			insertTestEntry(testFiles, ids);
						
			/***************************************************************/
			dao.getAuxFiles(pkgId);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageAuxFilesDAOSyb.PACKAGEID_KW + "=" + pkgId));
			ResultSet rs = stmt.getResultSet();
			for (String id: ids) {
				for (String auxFile: testFiles.get(id)) {
					assertTrue(rs.next());
					assertEquals((Integer) pkgId, (Integer) rs.getInt(PackageAuxFilesDAOSyb.PACKAGEID_KW));
					assertEquals(auxFile, rs.getString(PackageAuxFilesDAOSyb.AUXFILE_KW));
					assertEquals(id, rs.getString(PackageAuxFilesDAOSyb.FILEID_KW));
				}
			}
			assertFalse(rs.next());
				
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testGet3() {
		try {
			truncateTable();
			
			List<String> ids = testFileIds.subList(0, 50);
			
			insertTestEntry(testFiles, ids);
			
			/***************************************************************/
			dao.getAuxFiles(pkgId, ids);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageAuxFilesDAOSyb.PACKAGEID_KW + "=" + pkgId));
			ResultSet rs = stmt.getResultSet();
			for (String id: ids) {
				for (String auxFile: testFiles.get(id)) {
					assertTrue(rs.next());
					assertEquals((Integer) pkgId, (Integer) rs.getInt(PackageAuxFilesDAOSyb.PACKAGEID_KW));
					assertEquals(auxFile, rs.getString(PackageAuxFilesDAOSyb.AUXFILE_KW));
					assertEquals(id, rs.getString(PackageAuxFilesDAOSyb.FILEID_KW));
				}
			}
			assertFalse(rs.next());
				
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testInsert1() {
		try {
			truncateTable();
						
			/***************************************************************/
			dao.insertPackageAuxFile(pkgId, "FILEID0", "AUXNAME");
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageAuxFilesDAOSyb.PACKAGEID_KW + "=" + pkgId));
			ResultSet rs = stmt.getResultSet();
			assertTrue(rs.next());
			assertEquals((Integer) pkgId, (Integer) rs.getInt(PackageAuxFilesDAOSyb.PACKAGEID_KW));
			assertEquals("AUXNAME", rs.getString(PackageAuxFilesDAOSyb.AUXFILE_KW));
			assertEquals("FILEID0", rs.getString(PackageAuxFilesDAOSyb.FILEID_KW));
			assertFalse(rs.next());
				
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testInsert2() {
		try {
			truncateTable();
						
			/***************************************************************/
			dao.insertPackageAuxFiles(pkgId, testFiles);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageAuxFilesDAOSyb.PACKAGEID_KW + "=" + 
					pkgId + " ORDER BY " + PackageAuxFilesDAOSyb.FILEID_KW + " ASC, " + PackageAuxFilesDAOSyb.AUXFILE_KW + " ASC"));
			ResultSet rs = stmt.getResultSet();
			for (String id: testFileIds) {
				for (String auxFile: testFiles.get(id)) {
					assertTrue(rs.next());
					assertEquals((Integer) pkgId, (Integer) rs.getInt(PackageAuxFilesDAOSyb.PACKAGEID_KW));
					assertEquals(auxFile, rs.getString(PackageAuxFilesDAOSyb.AUXFILE_KW));
					assertEquals(id, rs.getString(PackageAuxFilesDAOSyb.FILEID_KW));
				}
			}
			assertFalse(rs.next());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testFind() {
		try {
			
			truncateTable();
			insertTestEntry(testFiles, testFileIds.subList(0, 1));
						
			String id = testFileIds.get(0);
			
			/***************************************************************/
			assertTrue(dao.findPackageAuxFile(pkgId, id, testFiles.get(id).get(0)));
			/****************************************************************/
		
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	@Test
	public void testInsertIfNotFound() {
		try {
			truncateTable();
			insertTestEntry(testFiles, testFileIds.subList(0, 10));
						
			/***************************************************************/
			Map<String, List<String>> packageFiles = new HashMap<String, List<String>>();
			
			List<String> ids = new ArrayList<String>();
			ids.add(testFileIds.get(3));
			ids.add(testFileIds.get(5));
			ids.add(testFileIds.get(8));
			ids.add(testFileIds.get(10));
			for (String id: ids)
				packageFiles.put(id, testFiles.get(id));
			
			dao.insertPackageAuxFilesIfNotFound(pkgId, packageFiles);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageAuxFilesDAOSyb.PACKAGEID_KW + "=" + 
					pkgId + " ORDER BY " + PackageAuxFilesDAOSyb.FILEID_KW + " ASC, " + PackageAuxFilesDAOSyb.AUXFILE_KW + " ASC"));
			ResultSet rs = stmt.getResultSet();
			for (String id: testFileIds.subList(0, 11)) {
				for (String auxFile: testFiles.get(id)) {
					assertTrue(rs.next());
					assertEquals((Integer) pkgId, (Integer) rs.getInt(PackageAuxFilesDAOSyb.PACKAGEID_KW));
					assertEquals(auxFile, rs.getString(PackageAuxFilesDAOSyb.AUXFILE_KW));
					assertEquals(id, rs.getString(PackageAuxFilesDAOSyb.FILEID_KW));
				}
			}
			assertFalse(rs.next());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testDelete1() {
		try {
			truncateTable();
			
			insertTestEntry(testFiles, testFileIds.subList(0, 1));
						
			String id = testFileIds.get(0);
			String auxFile = testFiles.get(id).get(0);
			/***************************************************************/
			dao.deletePackageAuxFile(pkgId, id, auxFile);
			/****************************************************************/
			
			String query = "SELECT * FROM " + DBTABLE + " WHERE " + PackageAuxFilesDAOSyb.PACKAGEID_KW + "=" + pkgId
			 + " AND " + PackageAuxFilesDAOSyb.FILEID_KW + "='" + id + "' AND " + PackageAuxFilesDAOSyb.AUXFILE_KW + "='" + auxFile + "'";  
			
			stmt.execute(query);
			ResultSet rs = stmt.getResultSet();
			assertFalse(rs.next());
				
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testDelete2() {
		try {
			truncateTable();
			
			insertTestEntry(testFiles, testFileIds);
			/***************************************************************/
			dao.deletePackageAuxFiles(pkgId, testFiles);
			/****************************************************************/
			
			String query = "SELECT * FROM " + DBTABLE + " WHERE " + PackageAuxFilesDAOSyb.PACKAGEID_KW + "=" + pkgId
			+ " AND " + PackageAuxFilesDAOSyb.FILEID_KW + " LIKE 'FILEID%' AND " + PackageAuxFilesDAOSyb.AUXFILE_KW + " LIKE 'AUXFILE%'";
			stmt.execute(query);
			ResultSet rs = stmt.getResultSet();
			assertFalse(rs.next());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	public static void truncateTable() throws Exception {
		stmt.execute("DELETE FROM " + DBTABLE + " where pkg_id=" + pkgId);
	}
	
	public void insertTestEntry(Map<String, List<String>> files, List<String> ids) throws Exception {
		for (String id: ids) {
			for (String auxFile: files.get(id)) {
				String query = "INSERT INTO " + DBTABLE + " (" + PackageAuxFilesDAOSyb.PACKAGEID_KW + "," + PackageAuxFilesDAOSyb.AUXFILE_KW + "," + 
				PackageAuxFilesDAOSyb.FILEID_KW + ") VALUES (" + pkgId + ", '" + auxFile + "', '" + id + "')";
				stmt.execute(query);
			}
		}
	}
	
	public void insertTestEntry(String id, String auxFile) throws Exception {
		String query = "INSERT INTO " + DBTABLE + " (" + PackageAuxFilesDAOSyb.PACKAGEID_KW + "," + PackageAuxFilesDAOSyb.AUXFILE_KW + "," + 
			PackageAuxFilesDAOSyb.FILEID_KW + ") VALUES (" + pkgId + ", '" + auxFile + "', '" + id + "')";
		stmt.execute(query);
	}
}
