/**
 * @author N. Kornweibel
 * @version 1.0
 *
 * An extension of the Report interface for generation of reports
 * from selected rows in a TableModel object.
 * 
 */
package org.eso.gasgano.datamodel.filesystem;

import java.io.PrintWriter;
import java.text.SimpleDateFormat;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.keyword.KeywordSet;
import org.eso.gasgano.tools.Report;


public class FitsFileReport extends Report {

    private String header;
    private String footer;
    private PrintWriter pw = null;
    private Vector inputFiles = null;
    private Vector keywordVector = null;
    private Vector colNames = null;
    private Hashtable keywords = null;

    private int columnWidths[] = null;

    public FitsFileReport(Vector fv, Vector kwv, Vector cn) {
        inputFiles = fv;
        keywordVector = kwv;
        colNames = cn;
        keywords = new Hashtable();

	if ( !keywordVector.elementAt(0).equals("FILENAME")) { 
	  keywordVector.insertElementAt("FILENAME",0);
	  colNames.insertElementAt("FILENAME",0);
	}
        Enumeration e = keywordVector.elements();
        while (e.hasMoreElements()) {
            Keyword k = new Keyword((String)e.nextElement(), "");
            keywords.put(k.getName(), k);
        }
        SimpleDateFormat formatter
            = new SimpleDateFormat("dd MMM yyyy hh:mm:ss");
        header = new String("\nReport Title: Fits File Report\n\n"
            +"Date Generated: "+formatter.format(new java.util.Date())+"\n");
        footer = new String("End of Report");

        // initialize the col widths with the widths of the col names.
        columnWidths = new int[keywordVector.size()];
        for (int i = 0; i < keywordVector.size(); i++) {
            columnWidths[i] = 0;
        }
    }

    public int[] getColumnWidths() {
        return columnWidths;
    }

    public void setHeader(String s) {
        header = s;
    }

    public void setFooter(String s) {
        footer = s;
    }

    public String getReportHeader() {
        return header;
    }

    public String getReportFooter() {
        return footer;
    }

    public void setPrintWriter(PrintWriter p) {
        pw = p;
    }

  public boolean generateReport() {
      String fileName = null;
    if (pw == null) {
      return false;
    }
    boolean rc = true;
    // pw.println(header);
    Enumeration e = inputFiles.elements();
    String values[][] = new String[inputFiles.size()][keywordVector.size()];
    int fileNum = 0;
    // here we loop over al the input files
    while (e.hasMoreElements()) {
      FitsFile ff = null;
      Object tmpObject = e.nextElement();
      KeywordSet ksoc  = (KeywordSet)tmpObject;
      boolean releaseMemory = true;

      if ( tmpObject instanceof FitsFile) { 
	ff = (FitsFile)tmpObject;
	fileName = ff.getFileName();
	if (ff.headerInMemory()) {
	  releaseMemory = false;
	} else {
	  ff.readFitsHeader();
	  ff.addMetaKeywords();
	}
      }
      int col = 0;
      Enumeration kwe = keywordVector.elements();
      // inner loop over the keywords
      while (kwe.hasMoreElements()) {
	  String tmpKeyword = (String)kwe.nextElement();
	  if ( tmpKeyword.equals("FILENAME")) { 
	      values[fileNum][col] = fileName;
	      storeWidth(fileName, col);
	  } else { 
	      Keyword k = ksoc.getKeyword(tmpKeyword);
	      if (k == null) { 
		  values[fileNum][col] = new String("");
	      } else {
		  storeWidth(k.getValue(), col);
		  values[fileNum][col] = k.getValue();
	      }
	  }
	  col++;
      }
      if (tmpObject instanceof FitsFile && releaseMemory) {
	ff.freeHeaderMemory();
      }
      fileNum++;
    }

    // print up the header.
    pw.println(header+"\n\n");
    e = colNames.elements();
    int colNum = 0;
    
    while (e.hasMoreElements()) {
      String val = (String)e.nextElement();
      storeWidth(val, colNum);
      pw.print(val);
      int yetToPad = 1+columnWidths[colNum]-val.length();
      while (yetToPad > 0) {
	pw.print(' ');
	yetToPad--;
      }
      colNum++;
    }
    pw.println("\n");
    
    // print the body
    for (int fn = 0; fn < inputFiles.size(); fn++) {
      for (int cn = 0; cn < keywordVector.size(); cn++) {
	String val = values[fn][cn];
	pw.print(val);
	int yetToPad = 1+columnWidths[cn]-val.length();
	while (yetToPad > 0) {
	  yetToPad--;
	  pw.print(' ');
	}
      }
      pw.println("");
    }
        // print the footer.
    pw.println(footer);

    if ( keywordVector.elementAt(0).equals("FILENAME")) { 
      keywordVector.removeElementAt(0 );
      colNames.removeElementAt( 0 );
    }

    return rc;
  }
  
  public void cleanup() {
    return;
  }
  
  public void storeWidth(String s, int c) {
    if (columnWidths != null && columnWidths[c] < s.length()) {
      columnWidths[c] = s.length();
    }
  }
  
}  // end of class FitsFileReport.
// end of file FitsFileReport.java.
