package org.eso.gasgano.datamodel.filesystem;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

/**
 * @author nkornwei
 *
 * File Selection Model containing a specific set of files. This model
 * is used as part of the data service of gasgano, to keep track of files added
 * to the data model through the data service.
 */
public class ServiceFileSet extends FileSelectionModel {

    /**
     * List of current File objects in the model.
     */
    List files = new ArrayList();
    
    /**
     * Create a ServiceFileSet 
     * @see java.lang.Object#Object()
     */
    public ServiceFileSet() {
        // nothing to do.
    }
    
    /**
     * Get all files in the model. If they do not exist they are removed
     * from the model. This check is performed everytime getFileList() is 
     * called.
     * @see org.eso.gasgano.datamodel.filesystem.FileSelectionModel#getFileList()
     */
    public Vector getFileList() {
        Vector rc = new Vector();
        List remainingFiles = new ArrayList();
        Iterator i = files.iterator();
        while (i.hasNext()) {
            File f = (File)i.next();
            if (f.exists()) {
                rc.add(f.getPath());
                remainingFiles.add(f);
            }
        }
        // keep the files that remain.
        files = remainingFiles;
        return rc;
    }

    /**
     * Add a file to the model - removes the file first
     * if present in the model.
     * @param file
     */
    public void addFile(File file) {
        removeFile(file);
        files.add(file);
    }
    
    
    /**
     * Remove a file from the model.
     * @param file
     */
    public void removeFile(File file) {
        files.remove(file);
    }

}
