/**
 * @author N. Kornweibel
 * @version 1.0
 */

package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

import org.eso.dfs.gui.SwingFileChooser;
import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.properties.PropertyDB;
import org.eso.gasgano.tools.DirUtils;

public class KeywordListLoader extends JPanel implements ActionListener {

    private KeywordListEditor editorPanel = null;
    private KeywordTable kwTable = null;
    private static final String openItem = "Load Keyword List";
    private JButton openButton = null;
    private JFileChooser fileChooser = null;

    public static void main(String[] args) {
        JFrame frame = new JFrame("Keyword Loader");
        KeywordListLoader loader = new KeywordListLoader();
        if (JOptionPane
            .showConfirmDialog(
                frame,
                loader,
                "Load keyword list",
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.PLAIN_MESSAGE)
            == JOptionPane.OK_OPTION) {
            Vector vec = loader.getKeywordTable().getKeywordList();
            for (int i = 0; i < vec.size(); i++) {
                System.out.println(vec.elementAt(i).toString());
            }
            vec = loader.getKeywordTable().getKeywordLabelList();
            for (int i = 0; i < vec.size(); i++) {
                System.out.println(vec.elementAt(i).toString());
            }
        }
        System.exit(0);
    }

    public KeywordListLoader() {
        super(new BorderLayout());
        kwTable = new KeywordTable();
        init();
    }

    public KeywordListLoader(String kwFile) {
        super(new BorderLayout());
        kwTable = new KeywordTable(kwFile);
        init();
    }

    private void init() {
        JPanel buttonPanel = new JPanel();
        editorPanel = new KeywordListEditor(kwTable);
        editorPanel.setReadOnly(true);
        add(editorPanel, BorderLayout.CENTER);
        openButton = new JButton(openItem);
        openButton.addActionListener(this);
        buttonPanel.add(openButton);
        add(buttonPanel, BorderLayout.SOUTH);
    }

    // implementation of the ActionListener interface
    public void actionPerformed(ActionEvent a) {
        if (a.getActionCommand() == openItem) {
            openFile();
        }
    }

    private JFileChooser getFileChooser() {
        if (fileChooser == null) {
            DFSDataModel dm = DFSDataModel.getDataModel();
            String kwDir =
                DirUtils.convertAbsoluteDir(
                    PropertyDB.getInstance().getProperty("KEYWORDLIST_DIR"));
            fileChooser = new SwingFileChooser(kwDir);
        } else {
            fileChooser.rescanCurrentDirectory();
        }
        return fileChooser;
    }

    public KeywordTable getKeywordTable() {
        return kwTable;
    }

    private void openFile() {
        JFileChooser openChooser = getFileChooser();
        openChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        openChooser.setDialogType(JFileChooser.OPEN_DIALOG);
        openChooser.setDialogTitle("Select keyword list file");
        openChooser.setApproveButtonText("Load");
        openChooser.setMultiSelectionEnabled(false);
        if (openChooser.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
            if (openChooser.getSelectedFile() != null) {
                String selectedFile = openChooser.getSelectedFile().getPath();
                if (!kwTable.readFile(selectedFile)) {
                    userMessage("Errors reading: " + selectedFile);
                }
            }
        }
    }

    public void userMessage(String msg) {
        JOptionPane.showMessageDialog(this, msg);
    }
} // end of class KeywordListLoader.
