/**
 * @author N. Kornweibel
 * @version 1.0
 *
 */
package org.eso.gasgano.keyword;

import java.io.Serializable;

public class ClassificationRule implements Serializable {

    static final long serialVersionUID = 5342691773597797000L;

    private String classification = null;
    private String instrument = null;
    private KeywordExpression rule = null;

    /**
     * @param s The name of the classification.
     */
    public void setClassification(String s) {
        classification = s;
    }

    /**
     * @return The name of the classification.
     */
    public String getClassification() {
        return classification;
    }

    /**
     * @param s The name of the instrument.
     */
    public void setInstrument(String s) {
        instrument = s;
    }

    /**
     * @return The name of the instrument.
     */
    public String getInstrument() {
        return instrument;
    }

    /**
     * @param n The KeywordExpression defining this calssification.
     */
    public void setRule(KeywordExpression n) {
        rule = n;
    }

    /**
     * @return The KeywordExpression defining this calssification.
     */
    public KeywordExpression getRule() {
        return rule;
    }

    /**
     * @param f The FitsFile to be classified.
     * @return True if the FitsFile matches this classification.
     */
    public boolean match(KeywordSet f) {
        boolean rc = true;
        if (f != null) {
            if (instrument != null) {
                Keyword instKw = f.getKeyword(Keyword.INSTRUMENT);
                if (instKw != null) {
                    String inst = instKw.getValue();
                    if (!inst.equalsIgnoreCase(instrument)) {
                        // System.out.println(f.toString()+": instrument no match");
                        rc = false;
                    }
                }
            }
        } else {
            rc = false;
        }
        if (rc && rule != null) {
            // pre-checks ok - continue to rule tree.
            // System.out.println(f.toString()+": start match");
            try {
                rc = rule.evaluate(f);
            } catch (ExpressionEvaluationException e) {
                System.out.println(e.toString());
                rc = false;
            }
            // System.out.println(f.toString()+": end match");
        }
        return rc;
    }

    public boolean equals(ClassificationRule cr) {
        if (getRule().toString().equals(cr.getRule().toString())
            && getInstrument().equals(cr.getInstrument())
            && getClassification().equals(cr.getClassification())) {
            return true;
        }
        return false;
    }

} // end of class ClassificationRule

// end of file ClassificationRule.java
