package org.eso.cpl;

/**
 * ScriptSyntax implementation corresponding to the Bourne shell.
 *
 * @author   Mark Taylor
 */
public class BourneSyntax implements ScriptSyntax {

    public String getName() {
        return "Bourne shell";
    }
 
    public String getExtension() {
        return ".sh";
    }

    public String startScript() {
        return "#!/bin/sh\n";
    }

    public String blankLine() {
        return "\n";
    }

    public String commentLine(String text) {
        return "# " + text + "\n";
    }

    public String invokeCommand(String cmd) {
        return cmd + "\n";
    }

    public String defineMacro(String name, String value) {
        return name + "=" + value + "\n";
    }

    public String defineMacro(String name, String[] value) {
        StringBuffer buf = new StringBuffer()
            .append(name)
            .append('=')
            .append('"');
        for (int i = 0; i < value.length; i++) {
            buf.append("\n   ")
               .append(value[i]);
        }
        buf.append('"')
           .append('\n');
        return buf.toString();
    }

    public String useMacro(String name) {
        return '$' + name;
    }

    public String quoteWord(String text) {
        StringBuffer buf = new StringBuffer();
        buf.append("'");
        for (int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);
            switch (c) {
                case '\'':
                   buf.append("'\"'\"'");
                   break;
                default:
                   buf.append(c);
            }
        }
        buf.append("'");
        return buf.toString();
    }

    public String namify(String name) {
        return name.replaceAll("[^a-zA-Z0-9]", "_");
    }
}
