package org.eso.cpl;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Represents the group that a {@link Frame} belongs to.
 * <p>
 * This corresponds to the <tt>cpl_frame_group</tt> enum in the
 * <tt>cpl_frame.h</tt> header file.  I don't know anywhere its semantics
 * are documented.
 * <p>
 * This class exemplifies the typesafe enum pattern.
 *
 * @author   Mark Taylor
 */
public class FrameGroup {

    private final String name_;
    private final String idName_;

    /**
     * Constructs a new FrameGroup object.
     * This private constructor is only used to create static final members
     * of this class.
     *
     * @param  name  the public name of this object
     * @param  idName  the name of the <tt>cpl_frame_group</tt> enum member
     *         to which it corresponds (may be used by native code)
     */
    private FrameGroup(String name, String idName) {
        name_ = name;
        idName_ = idName;
    }

    public String getName() {
        return name_;
    }

    public String toString() {
        return getName();
    }

    public static final FrameGroup RAW =
        new FrameGroup("RAW", "CPL_FRAME_GROUP_RAW");
    public static final FrameGroup CALIB = 
        new FrameGroup("CALIB", "CPL_FRAME_GROUP_CALIB");
    public static final FrameGroup PRODUCT = 
        new FrameGroup("PRODUCT", "CPL_FRAME_GROUP_PRODUCT");

    /** Unmodifiable list containing all the instances of this class.  */
    public static final List ALL_GROUPS =
        Collections.unmodifiableList(Arrays.asList(new FrameGroup[] {
            RAW, CALIB, PRODUCT,
        }));

}
