package org.eso.cpl;

import java.util.HashSet;
import java.util.Set;

/**
 * ScriptSyntax implementation corresponding to Perl.
 * 
 * @author   Mark Taylor
 */
public class PerlSyntax implements ScriptSyntax {

    Set vars_ = new HashSet();

    public String getName() {
        return "Perl";
    }

    public String getExtension() {
        return ".pl";
    }

    public String startScript() {
        return "#!/usr/bin/perl -w\nuse strict;\n";
    }

    public String blankLine() {
        return "\n";
    }

    public String commentLine(String text) {
        return "# " + text + "\n";
    }

    public String invokeCommand(String cmd) {
        return "system(\"" + cmd + "\");\n";
    }

    public String defineMacro(String name, String value) {
        StringBuffer buf = new StringBuffer(declareMy(name));
        buf.append("my $" + name + " = " + value + ";\n");
        return buf.toString();
    }

    public String defineMacro(String name, String[] value) {
        StringBuffer buf = new StringBuffer(declareMy(name));
        buf.append('$')
           .append(name)
           .append(" = ")
           .append("qq{");
        for (int i = 0; i < value.length; i++) {
            buf.append("\n   ")
               .append(value[i]);
        }
        buf.append('\n')
           .append("};\n");
        return buf.toString();
    }

    public String useMacro(String name) {
        return '$' + name;
    }

    public String quoteWord(String text) {
        StringBuffer buf = new StringBuffer();
        buf.append("'");
        for (int i = 0; i < text.length(); i++) {
            char c = text.charAt(i);
            switch (c) {
                case '\'':
                    buf.append('\\')
                       .append('\'');
                    break;
                default:
                    buf.append(c);
            }
        }
        buf.append('\'');
        return buf.toString();
    }
         
    public String namify(String name) {
        return name.replaceAll("[^a-zA-Z0-9]", "_");
    }

    private String declareMy(String name) {
        String dec;
        if (vars_.contains(name)) {
            dec = "";
        }
        else {
            dec = "my $" + name + ";\n";
            vars_.add(name);
        }
        return dec;
    }
}
