package org.eso.cpl;

/**
 * Interface for libraries which can hold {@link Recipe} objects.
 *
 * @author   Mark Taylor
 */
public interface RecipeLibrary {

    /**
     * Returns the recipes held by this library.  The returned array may
     * be empty, but should not be <tt>null</tt>.
     * 
     * @return  array of recipes
     */
    Recipe[] getRecipes();

    /**
     * Returns an array of objects held by this library which are not 
     * functional Recipe objects.  If the library can only hold recipes,
     * this may return an empty array or <tt>null</tt>.
     * The objects in the returned array can be anything, but they ought
     * to have a sensible <tt>toString</tt> method.
     *
     * @return  array of non-recipes contained by this library, or <tt>null</tt>
     */
    Object[] getOtherItems();

    /**
     * Returns the name of this library.
     *
     * @return   name
     */
    String getName();

    /**
     * Returns the location of this library, if known.  If applicable,
     * this may be a full pathname or URL for the library in question.
     *
     * @return  library location
     */
    String getLocation();
}
