package org.eso.cpl.gui;

import java.io.BufferedInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import javax.swing.Icon;
import javax.swing.ImageIcon;

/**
 * Class which provides icons used by the CPL GUI.
 * The icons are available as static members of this class.
 */
public class Icons {

    public static final Icon ARROW_EES = makeIcon("arrow_ees.gif");
    public static final Icon ARROW_ES = makeIcon("arrow_es.gif");
    public static final Icon ARROW_SE = makeIcon("arrow_se.gif");
    public static final Icon ARROW_S = makeIcon("arrow_s.gif");
    public static final Icon ARROW_WS = makeIcon("arrow_ws.gif");
    public static final Icon EXECOK = makeIcon("execok.gif");
    public static final Icon EXECERR = makeIcon("execfail.gif");
    public static final Icon EXECBLANK = makeIcon("execblank.gif");

    private static final String PREFIX = "/org/eso/cpl/images/";
        
    /**
     * Constructs an icon given its filename relative to the class path
     * given by PREFIX.
     *
     * @param  filename  icon source relative location
     * @return  icon object for <tt>filename</tt>
     */
    private static Icon makeIcon(String filename) {
        String loc = PREFIX + filename;
        URL url = Icons.class.getClass().getResource(loc);
        if (url == null) {
            throw new RuntimeException("Resource not found: " + loc);
        }
        try {
            InputStream istrm = url.openStream();
            istrm = new BufferedInputStream(istrm);
            ByteArrayOutputStream ostrm = new ByteArrayOutputStream();
            for (int b = -1; (b = istrm.read()) > -1; ) {
                ostrm.write(b);
            }
            istrm.close();
            ostrm.close();
            return new ImageIcon(ostrm.toByteArray());
        }
        catch (IOException e) {
            throw new RuntimeException("Error loading icon " + url, e);
        }
    }
}
